@@ -1,7 +1,8 @@
-/*      $Id: shortcuts_plugin.c 20983 2006-04-16 09:15:54Z benny $
+/*      $Id: shortcuts_plugin.c 22152 2006-06-18 16:17:04Z pollux $
  *
  * Copyright (c) 2006 Jean-Francois Wauthy <pollux@xfce.org>
  *                    Olivier Fourdan <fourdan@xfce.org>
+ *                    Benedikt Meurer <benny@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,7 +90,6 @@ static gchar *theme_name = NULL;
 static gchar *theme_path = NULL;
 
 /* static prototypes */
-static void run_dialog (McsPlugin *);
 gboolean shortcuts_plugin_save_settings (McsPlugin *);
 static void cb_browse_command (GtkWidget *, GtkEntry *);
 static gboolean command_exists (const gchar *);
@@ -134,6 +134,7 @@ handleKeyPress (XKeyPressedEvent * ev)
                     g_error_free (error);
                 }
             }
+
             return;
         }
     }
@@ -163,6 +164,7 @@ handleMappingNotify (XMappingEvent * ev)
     }
 
     /* Regrab all keys if the notify is for keyboard (ie not pointer) */
+    gdk_error_trap_push ();
     if (ev->request != MappingPointer)
     {
         for (element = shortcut_list; element != NULL; element = g_slist_next (element))
@@ -172,6 +174,8 @@ handleMappingNotify (XMappingEvent * ev)
             grab_key (shortcut->key);
         }
     }
+    gdk_flush();
+    gdk_error_trap_pop ();
 }
 
 static void
@@ -193,37 +197,22 @@ handle_event (XEvent * ev)
     }
 }
 
-static XfceFilterStatus
-event_filter (XEvent * xevent, gpointer data)
+static GdkFilterReturn
+event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 {
-    handle_event (xevent);
-    return XEV_FILTER_STOP;
+    handle_event ((XEvent *) gdk_xevent);
+
+    return GDK_FILTER_CONTINUE;
 }
 
-void
+static void
 add_event_listener (void)
 {
-    GdkScreen *gscr;
     GdkDisplay *gdisplay;
-    GdkWindow *event_win;
-    GtkWidget *gtk_win;
-    XfceFilterSetup *xfilter;
 
     gdisplay = gdk_display_get_default ();
-    gscr = gdk_display_get_default_screen (gdisplay);
-
-    gtk_win = gtk_window_new (GTK_WINDOW_POPUP);
-    gtk_window_set_screen (GTK_WINDOW (gtk_win), gscr);
-    gtk_window_resize (GTK_WINDOW (gtk_win), 5, 5);
-    gtk_window_move (GTK_WINDOW (gtk_win), -1000, -1000);
-    gtk_widget_show (GTK_WIDGET (gtk_win));
-
     XAllowEvents (GDK_DISPLAY_XDISPLAY (gdisplay), AsyncBoth, CurrentTime);
-    event_win = xfce_add_event_win (gscr, KeyPressMask | KeyReleaseMask);
-    gdk_window_set_user_data (event_win, gtk_win);
-
-    xfilter = xfce_init_event_filter (NULL);
-    xfce_push_event_filter (xfilter, event_filter, (gpointer) NULL);
+    gdk_window_add_filter (NULL, event_filter, (gpointer) NULL);
 }
 
 static void
@@ -268,8 +257,8 @@ search_key_in_shortcut_list (gchar * key_string)
             return (element);
         }
     }
-
     g_free (key);
+
     return (NULL);
 }
 
@@ -314,8 +303,6 @@ get_theme_name_start (GMarkupParseContext * context, const gchar * element_name,
             state->theme_name = g_strdup (attribute_values[i]);
 
     }
-    else if (!state->started)
-        return;
 }
 
 static void
@@ -367,6 +354,7 @@ get_theme_name (const gchar * filename)
             g_warning ("Unable to read file '%s' (%d): %s\n", filename, err->code, err->message);
             g_error_free (err);
         }
+
         goto cleanup;
     }
 
@@ -379,6 +367,7 @@ get_theme_name (const gchar * filename)
     {
         g_warning ("Error parsing shortcuts definitions (%d): %s\n", err->code, err->message);
         g_error_free (err);
+
         goto cleanup;
     }
 
@@ -493,11 +482,15 @@ parse_theme (const gchar * filename, KeyboardMcsDialog * dialog)
 #ifdef HAVE_MMAP
     fd = open (filename, O_RDONLY, 0);
     if (fd < 0)
+    {
         goto cleanup;
+    }
 
     maddr = mmap (NULL, st.st_size, PROT_READ, MAP_FILE | MAP_SHARED, fd, 0);
     if (maddr)
+    {
         file_contents = maddr;
+    }
 #endif
 
     if (!file_contents && !g_file_get_contents (filename, &file_contents, NULL, &err))
@@ -522,30 +515,39 @@ parse_theme (const gchar * filename, KeyboardMcsDialog * dialog)
     {
         state.treeview = NULL;
     }
-        
 
     if (shortcut_list)
+    {
         free_shortcut_list ();
+    }
 
     gpcontext = g_markup_parse_context_new (&gmparser, 0, &state, NULL);
 
+    gdk_error_trap_push ();
     if (!g_markup_parse_context_parse (gpcontext, file_contents, st.st_size, &err))
     {
         g_warning ("Error parsing shortcuts definitions (%d): %s\n", err->code, err->message);
         g_error_free (err);
+
         goto cleanup;
     }
 
     if (g_markup_parse_context_end_parse (gpcontext, NULL))
     {
         if (dialog)
+        {
             dialog->theme_modified = FALSE;
+        }
         ret = TRUE;
     }
 
   cleanup:
+    gdk_flush();
+    gdk_error_trap_pop ();
     if (gpcontext)
+    {
         g_markup_parse_context_free (gpcontext);
+    }
 #ifdef HAVE_MMAP
     if (maddr)
     {
@@ -553,17 +555,23 @@ parse_theme (const gchar * filename, KeyboardMcsDialog * dialog)
         file_contents = NULL;
     }
     if (fd > -1)
+    {
         close (fd);
+    }
 #endif
     if (file_contents)
+    {
         free (file_contents);
+    }
     if (state.parents)
     {
         g_queue_foreach (state.parents, (GFunc) g_free, NULL);
         g_queue_free (state.parents);
     }
     if (state.theme_name)
+    {
         g_free (state.theme_name);
+    }
 
     return ret;
 }
@@ -571,16 +579,18 @@ parse_theme (const gchar * filename, KeyboardMcsDialog * dialog)
 static gboolean
 save_theme_foreach_func (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
 {
-    gchar *command;
-    gchar *shortcut;
+    gchar *command = NULL, *escaped_command = NULL;
+    gchar *shortcut = NULL;
     FILE *file_theme;
 
     gtk_tree_model_get (model, iter, COLUMN_COMMAND, &command, COLUMN_SHORTCUT, &shortcut, -1);
 
     file_theme = (FILE *) data;
 
-    fprintf (file_theme, "\t<shortcut command=\"%s\" keys=\"%s\"/>\n", command, shortcut ? shortcut : "");
+    escaped_command = g_markup_escape_text (command, -1);
+    fprintf (file_theme, "\t<shortcut command=\"%s\" keys=\"%s\"/>\n", escaped_command, shortcut ? shortcut : "");
 
+    g_free (escaped_command);
     g_free (command);
     g_free (shortcut);
 
@@ -664,10 +674,7 @@ void shortcuts_plugin_response (gint response_id, KeyboardMcsDialog* status)
     if(response_id == GTK_RESPONSE_HELP)
     {
         g_message("HELP: TBD");
-        return;
     }
-
-    /*gtk_widget_destroy (dialog);*/
 }
 
 static void
@@ -726,7 +733,9 @@ update_themes_list (KeyboardMcsDialog * dialog)
                     }
                 }
                 else
+                {
                     g_warning ("Error while loading themes : %s has no name", full_path);
+                }
 
                 g_free (full_path);
                 g_free (theme);
@@ -870,23 +879,6 @@ cb_treeview_themes_activate (GtkWidget * treeview, GtkTreePath * path, GtkTreeVi
     }
 }
 
-static void
-cb_popup_rename_theme (GtkWidget * widget, gpointer data)
-{
-    KeyboardMcsDialog * dialog = (KeyboardMcsDialog *) data;
-    
-    if (strcmp(DEFAULT_SHORTCUT_THEME, theme_name))
-    {
-        show_rename_theme_dialog (dialog);
-    }
-}
-
-static void
-show_edit_shortcut_dialog (KeyboardMcsDialog * dialog)
-{
-}
-
-/* shortcut edition */
 static gboolean
 is_modifier (guint keycode)
 {
@@ -925,63 +917,87 @@ shortcut_tree_foreach_func (GtkTreeModel * model, GtkTreePath * path, GtkTreeIte
     }
 
     g_free (current_shortcut);
+
     return stfs->found;
 }
 
 static gboolean
 cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
 {
-    KeyboardMcsDialog *dialog;
+    KeyboardMcsDialog *dialog = (KeyboardMcsDialog *) data;
     GdkModifierType consumed_modifiers = 0;
     gchar shortcut_string[80] = "";
     guint keyval;
+    guint modifiers;
     gchar *accelerator;
-
-    gint i;
     gchar **shortcuts;
     gchar **current_shortcut;
     shortcut_tree_foreach_struct stfs;
-
     GtkTreeModel *model;
     GtkTreeSelection *selection;
     GtkTreeIter iter;
-
     GSList *element;
     gchar *command;
 
     if (is_modifier (event->hardware_keycode))
         return TRUE;
 
-    dialog = (KeyboardMcsDialog *) data;
+    gdk_keymap_translate_keyboard_state (gdk_keymap_get_default (), 
+        event->hardware_keycode, event->state, event->group, NULL, NULL, NULL, &consumed_modifiers);
 
-    gdk_keymap_translate_keyboard_state (gdk_keymap_get_default (), event->hardware_keycode, event->state, event->group, NULL, NULL, NULL, &consumed_modifiers);
     keyval = gdk_keyval_to_lower (event->keyval);
 
-    if (keyval == GDK_ISO_Left_Tab)
-        keyval = GDK_Tab;
+    switch (keyval)
+    {
+        case GDK_ISO_Left_Tab:
+            keyval = GDK_Tab;
+            break;
+        case GDK_ISO_Level3_Latch:
+        case GDK_ISO_Level3_Lock:
+        case GDK_ISO_Level3_Shift:
+        case GDK_Scroll_Lock:
+            return TRUE;
+            break;
+        default:
+            break;
+    }
 
     /* Release keyboard */
     gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 
-    accelerator = gtk_accelerator_name (keyval, event->state);
+    /*
+     * gtk_accelerator_get_default_mod_mask () limits the number of modifiers.
+     * That's a good thing because we don't want the "fake" modifiers such as
+     * Scroll/Num/Caps Lock to be taken into account.
+     *
+     * Unfortunately, at this time of writing, gdk doesn't know about all of the
+     * modifiers, like META, SUPER or HYPER, but just SHIFT, ALT and CONTROL...
+     *
+     * It means that ppl won't be able to use the "Windows" key as a modifier...
+     * Too bad, that's life, you may gently ask the gtk+ maintainers about it.
+     *
+     * Things may change in the future? See this thread:
+     *
+     * http://mail.gnome.org/archives/gtk-devel-list/2005-September/msg00024.html
+     *
+     * (Olivier)
+     */
 
-    for (i = 0; i < strlen (accelerator); i++)
-    {
-        if (accelerator[i] == '>')
-            accelerator[i] = '<';
-    }
+    modifiers = event->state & (~consumed_modifiers | GDK_MODIFIER_MASK);
+    modifiers = modifiers & gtk_accelerator_get_default_mod_mask ();
+    accelerator = gtk_accelerator_name (keyval, modifiers);
 
-    shortcuts = g_strsplit (accelerator, "<", 0);
+    shortcuts = g_strsplit_set (accelerator, "<>", 0);
 
     current_shortcut = shortcuts;
     while (*current_shortcut)
     {
-        if (strlen (*current_shortcut) > 0 && (strcmp (*current_shortcut, "Mod2") != 0))
+        if (strlen (*current_shortcut))
         {
             strcat (shortcut_string, *current_shortcut);
             strcat (shortcut_string, "+");
         }
-        *current_shortcut++;
+        current_shortcut++;
     }
 
     shortcut_string[strlen (shortcut_string) - 1] = '\0';       /* replace the trailing '+' */
@@ -1024,6 +1040,7 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
 
     gtk_tree_model_get (model, &iter, COLUMN_COMMAND, &command, -1);
 
+    gdk_error_trap_push ();
     if ((element = search_key_in_shortcut_list (shortcut_string)) != NULL)
     {
         /* Change the command associated with the shortcut */
@@ -1045,12 +1062,14 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
         grab_key (launcher_data->key);
         shortcut_list = g_slist_append (shortcut_list, launcher_data);
     }
-
+    gdk_flush();
+    gdk_error_trap_pop ();
     g_free (command);
 
     /* End operations */
     dialog->theme_modified = TRUE;
     gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_OK);
+
     return TRUE;
 }
 
@@ -1192,7 +1211,7 @@ cb_treeview_shortcuts_activate (GtkWidget * treeview, GtkTreePath * path, GtkTre
         g_signal_connect (G_OBJECT (dialog_compose), "key-release-event", G_CALLBACK (cb_compose_shortcut), dialog);
 
         /* Take control on the keyboard */
-        if (gdk_keyboard_grab (gtk_widget_get_root_window (label), TRUE, GDK_CURRENT_TIME) != GDK_GRAB_SUCCESS)
+        if (gdk_keyboard_grab (gtk_widget_get_root_window (dialog_compose), TRUE, GDK_CURRENT_TIME) != GDK_GRAB_SUCCESS)
         {
             g_warning ("Cannot grab the keyboard");
             g_free (dialog_text);
@@ -1201,9 +1220,15 @@ cb_treeview_shortcuts_activate (GtkWidget * treeview, GtkTreePath * path, GtkTre
             return;
         }
 
+        /* Stop our own event filter so that it doesn's interfere with gtk+ */
+        gdk_window_remove_filter (NULL, event_filter, (gpointer) NULL);
+
         /* Show dialog */
         response = gtk_dialog_run (GTK_DIALOG (dialog_compose));
 
+        /* Put back our event filter */
+        gdk_window_add_filter (NULL, event_filter, (gpointer) NULL);
+
         if (response == GTK_RESPONSE_NO)
         {
             gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_SHORTCUT, "", -1);
@@ -1218,13 +1243,6 @@ cb_treeview_shortcuts_activate (GtkWidget * treeview, GtkTreePath * path, GtkTre
         g_free (command);
         g_free (shortcut);
     }
-
-}
-
-static void
-cb_popup_edit_shortcut (GtkWidget * widget, gpointer data)
-{
-    show_edit_shortcut_dialog (data);
 }
 
 static gboolean
@@ -1275,6 +1293,7 @@ cb_popup_menu_themes (GtkTreeView * treeview, GdkEventButton * event, gpointer d
         screen = xfce_gdk_display_locate_monitor_with_pointer (NULL, NULL);
         gtk_menu_set_screen (GTK_MENU (dialog->menu_popup_themes), screen ? screen : gdk_screen_get_default ());
         gtk_menu_popup (GTK_MENU (dialog->menu_popup_themes), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time ());
+
         return TRUE;
     }
 
@@ -1314,6 +1333,7 @@ cb_popup_menu_shortcuts (GtkTreeView * treeview, GdkEventButton * event, gpointe
         screen = xfce_gdk_display_locate_monitor_with_pointer (NULL, NULL);
         gtk_menu_set_screen (GTK_MENU (dialog->menu_popup_shortcuts), screen ? screen : gdk_screen_get_default ());
         gtk_menu_popup (GTK_MENU (dialog->menu_popup_shortcuts), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time ());
+
         return TRUE;
     }
 
@@ -1492,18 +1512,6 @@ cb_button_del_theme_clicked (GtkWidget * widget, gpointer data)
     del_theme ((KeyboardMcsDialog *) data);
 }
 
-static void
-cb_menuitem_add_theme_activate (GtkWidget * widget, gpointer data)
-{
-    add_theme ((KeyboardMcsDialog *) data);
-}
-
-static void
-cb_menuitem_del_theme_activate (GtkWidget * widget, gpointer data)
-{
-    del_theme ((KeyboardMcsDialog *) data);
-}
-
 static void
 cb_button_del_shortcut_clicked (GtkWidget * widget, gpointer data)
 {
@@ -1603,20 +1611,93 @@ cb_button_add_shortcut_clicked (GtkWidget * widget, gpointer data)
         {
             GtkTreeModel *model;
             GtkTreeIter iter;
-
+            GtkTreeViewColumn *column;
+            GtkTreePath *tree_path;
+            
             model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->treeview_shortcuts));
             gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
             gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_COMMAND, 
                                 gtk_entry_get_text (GTK_ENTRY (entry)), COLUMN_SHORTCUT, "", -1);
+            column = gtk_tree_view_get_column (GTK_TREE_VIEW (dialog->treeview_shortcuts), COLUMN_SHORTCUT);
+            tree_path = gtk_tree_model_get_path(model, &iter);
+            gtk_tree_view_set_cursor(GTK_TREE_VIEW (dialog->treeview_shortcuts), tree_path, column, FALSE);
+            cb_treeview_shortcuts_activate (dialog->treeview_shortcuts, tree_path, column, data);
+            gtk_tree_path_free(tree_path);
+
             dialog->theme_modified = TRUE;
         }
         else
+        {
             xfce_err (_("The command doesn't exist or the file is not executable !"));
+        }
     }
 
     gtk_widget_destroy (dialog_command);
 }
 
+static void
+cb_menuitem_add_theme_activate (GtkWidget * widget, gpointer data)
+{
+    add_theme ((KeyboardMcsDialog *) data);
+}
+
+static void
+cb_menuitem_del_theme_activate (GtkWidget * widget, gpointer data)
+{
+    del_theme ((KeyboardMcsDialog *) data);
+}
+
+static void
+cb_menuitem_rename_theme_activate (GtkWidget * widget, gpointer data)
+{
+    KeyboardMcsDialog * dialog = (KeyboardMcsDialog *) data;
+    
+    if (strcmp(DEFAULT_SHORTCUT_THEME, theme_name))
+    {
+        show_rename_theme_dialog (dialog);
+    }
+}
+
+static void
+cb_menuitem_edit_shortcut_activate (GtkWidget * widget, gpointer data)
+{
+    KeyboardMcsDialog * dialog = (KeyboardMcsDialog *) data;
+
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    GtkTreeViewColumn *column;
+    GtkTreePath *tree_path;
+    GtkTreeSelection *selection;
+
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->treeview_shortcuts));
+    if (!selection)
+    {
+        return;
+    }
+
+    gtk_tree_selection_get_selected (selection, &model, &iter);
+    tree_path = gtk_tree_model_get_path(model, &iter);
+    column = gtk_tree_view_get_column (GTK_TREE_VIEW (dialog->treeview_shortcuts), COLUMN_COMMAND);
+    cb_treeview_shortcuts_activate (dialog->treeview_shortcuts, tree_path, column, data);
+    gtk_tree_path_free(tree_path);
+}
+
+static void
+cb_menuitem_add_shortcut_activate (GtkWidget * widget, gpointer data)
+{
+    KeyboardMcsDialog * dialog = (KeyboardMcsDialog *) data;
+
+    cb_button_add_shortcut_clicked (dialog->button_add_shortcut, data);
+}
+
+static void
+cb_menuitem_del_shortcut_activate (GtkWidget * widget, gpointer data)
+{
+    KeyboardMcsDialog * dialog = (KeyboardMcsDialog *) data;
+
+    cb_button_del_shortcut_clicked (dialog->button_del_shortcut, data);
+}
+
 /********************/
 /* Shortcut edition */
 /********************/
@@ -1631,41 +1712,124 @@ command_exists (const gchar * command)
     cmd_tok = strtok (cmd_buf, " ");
 
     if (g_find_program_in_path (cmd_buf) != NULL)
+    {
         result = TRUE;
-
+    }
     g_free (cmd_buf);
 
     return result;
 }
 
 static void
-cb_browse_command (GtkWidget * widget, GtkEntry * entry_command)
-{
-    GtkWidget *filesel_dialog;
+cb_browse_command (GtkWidget * widget, GtkEntry * command_entry)
+{    
+  GtkFileFilter *filter;
+  GtkWidget     *chooser;
+  gchar         *filename;
+  gchar         *s;
 
-    filesel_dialog = xfce_file_chooser_new (_("Select command"), NULL,
-                                            XFCE_FILE_CHOOSER_ACTION_OPEN, 
-                                            GTK_STOCK_CANCEL, 
-                                            GTK_RESPONSE_CANCEL, 
-                                            GTK_STOCK_OPEN, 
-                                            GTK_RESPONSE_ACCEPT, 
-                                            NULL);
+  chooser = gtk_file_chooser_dialog_new (_("Select an Application"),
+                                         NULL,
+                                         GTK_FILE_CHOOSER_ACTION_OPEN,
+                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                         NULL);
+  gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), TRUE);
 
-    xfce_file_chooser_set_filename (XFCE_FILE_CHOOSER (filesel_dialog), 
-                                    gtk_entry_get_text (entry_command));
-    xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (filesel_dialog));
+  /* add file chooser filters */
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, _("All Files"));
+  gtk_file_filter_add_pattern (filter, "*");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
 
-    if (gtk_dialog_run (GTK_DIALOG (filesel_dialog)) == GTK_RESPONSE_ACCEPT)
-    {
-        gchar *filename;
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, _("Executable Files"));
+  gtk_file_filter_add_mime_type (filter, "application/x-csh");
+  gtk_file_filter_add_mime_type (filter, "application/x-executable");
+  gtk_file_filter_add_mime_type (filter, "application/x-perl");
+  gtk_file_filter_add_mime_type (filter, "application/x-python");
+  gtk_file_filter_add_mime_type (filter, "application/x-ruby");
+  gtk_file_filter_add_mime_type (filter, "application/x-shellscript");
+  gtk_file_filter_add_pattern (filter, "*.pl");
+  gtk_file_filter_add_pattern (filter, "*.py");
+  gtk_file_filter_add_pattern (filter, "*.rb");
+  gtk_file_filter_add_pattern (filter, "*.sh");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
+  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (chooser), filter);
 
-        filename = xfce_file_chooser_get_filename (XFCE_FILE_CHOOSER (filesel_dialog));
-        gtk_entry_set_text (entry_command, filename);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, _("Perl Scripts"));
+  gtk_file_filter_add_mime_type (filter, "application/x-perl");
+  gtk_file_filter_add_pattern (filter, "*.pl");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
 
-        g_free (filename);
-    }
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, _("Python Scripts"));
+  gtk_file_filter_add_mime_type (filter, "application/x-python");
+  gtk_file_filter_add_pattern (filter, "*.py");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
 
-    gtk_widget_destroy (filesel_dialog);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, _("Ruby Scripts"));
+  gtk_file_filter_add_mime_type (filter, "application/x-ruby");
+  gtk_file_filter_add_pattern (filter, "*.rb");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, _("Shell Scripts"));
+  gtk_file_filter_add_mime_type (filter, "application/x-csh");
+  gtk_file_filter_add_mime_type (filter, "application/x-shellscript");
+  gtk_file_filter_add_pattern (filter, "*.sh");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
+
+  /* use the bindir as default folder */
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), BINDIR);
+
+  /* setup the currently selected file */
+  filename = gtk_editable_get_chars (GTK_EDITABLE (command_entry), 0, -1);
+  if (G_LIKELY (filename != NULL))
+  {
+      /* use only the first argument */
+      s = strchr (filename, ' ');
+      if (G_UNLIKELY (s != NULL))
+      {
+          *s = '\0';
+      }
+
+      /* check if we have a file name */
+      if (G_LIKELY (*filename != '\0'))
+      {
+          /* check if the filename is not an absolute path */
+          if (G_LIKELY (!g_path_is_absolute (filename)))
+          {
+              /* try to lookup the filename in $PATH */
+              s = g_find_program_in_path (filename);
+              if (G_LIKELY (s != NULL))
+              {
+                  /* use the absolute path instead */
+                  g_free (filename);
+                  filename = s;
+              }
+          }
+      
+          /* check if we have an absolute path now */
+          if (G_LIKELY (g_path_is_absolute (filename)))
+              gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser), filename);
+      }
+
+      /* release the filename */
+      g_free (filename);
+  }
+
+  /* run the chooser dialog */
+  if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT)
+  {
+      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+      gtk_entry_set_text (GTK_ENTRY (command_entry), filename);
+      g_free (filename);
+  }
+
+  gtk_widget_destroy (chooser);
 }
 
 GtkWidget* shortcuts_plugin_create_dialog (KeyboardMcsDialog *dialog)
@@ -1825,12 +1989,15 @@ void shortcuts_plugin_setup_dialog (KeyboardMcsDialog * dialog)
     g_signal_connect (G_OBJECT (dialog->button_del_shortcut), "clicked", G_CALLBACK (cb_button_del_shortcut_clicked), dialog);
 
     /* popup menus */
-    g_signal_connect (G_OBJECT (dialog->menuitem_popup_rename_theme), "activate", G_CALLBACK (cb_popup_rename_theme), dialog);
     g_signal_connect (G_OBJECT (dialog->menuitem_popup_add_theme), "activate", G_CALLBACK (cb_menuitem_add_theme_activate), dialog);
     g_signal_connect (G_OBJECT (dialog->menuitem_popup_del_theme), "activate", G_CALLBACK (cb_menuitem_del_theme_activate), dialog);
+    g_signal_connect (G_OBJECT (dialog->menuitem_popup_rename_theme), "activate", G_CALLBACK (cb_menuitem_rename_theme_activate), dialog);
     g_signal_connect (G_OBJECT (dialog->treeview_themes), "row-activated", G_CALLBACK (cb_treeview_themes_activate), dialog);
 
-    g_signal_connect (G_OBJECT (dialog->menuitem_popup_edit_shortcut), "activate", G_CALLBACK (cb_popup_edit_shortcut), dialog);
+    g_signal_connect (G_OBJECT (dialog->menuitem_popup_add_shortcut), "activate", G_CALLBACK (cb_menuitem_add_shortcut_activate), dialog);
+    g_signal_connect (G_OBJECT (dialog->menuitem_popup_del_shortcut), "activate", G_CALLBACK (cb_menuitem_del_shortcut_activate), dialog);
+    g_signal_connect (G_OBJECT (dialog->menuitem_popup_edit_shortcut), "activate", G_CALLBACK (cb_menuitem_edit_shortcut_activate), dialog);
+
     g_signal_connect (G_OBJECT (dialog->treeview_shortcuts), "row-activated", G_CALLBACK (cb_treeview_shortcuts_activate), dialog);
 
     update_themes_list (dialog);
