@@ -220,21 +220,16 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     dialog->mcs_plugin = mcs_plugin;
     dialog->radiobutton1_group = NULL;
 
-    dialog->mouse_dialog = gtk_dialog_new();
+    dialog->mouse_dialog = xfce_titled_dialog_new();
 
-    gtk_window_set_icon(GTK_WINDOW(dialog->mouse_dialog), mcs_plugin->icon);
+    gtk_window_set_icon_name(GTK_WINDOW(dialog->mouse_dialog), "xfce4-mouse");
 
-    gtk_window_set_title(GTK_WINDOW(dialog->mouse_dialog), _("Mouse"));
+    gtk_window_set_title(GTK_WINDOW(dialog->mouse_dialog), _("Mouse Preferences"));
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog->mouse_dialog), FALSE);
 
     dialog->dialog_vbox1 = GTK_DIALOG(dialog->mouse_dialog)->vbox;
     gtk_widget_show(dialog->dialog_vbox1);
 
-    dialog->dialog_header = xfce_create_header(mcs_plugin->icon,
-                                               _("Mouse Preferences"));
-    gtk_widget_show(dialog->dialog_header);
-    gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->dialog_header, FALSE, TRUE, 0);
-    
     dialog->notebook_vbox = gtk_vbox_new(FALSE, 6);
     gtk_container_set_border_width(GTK_CONTAINER(dialog->notebook_vbox), 6);
     gtk_widget_show(dialog->notebook_vbox);
@@ -457,10 +452,13 @@ mcs_plugin_init(McsPlugin *mcs_plugin)
 
     create_channel(mcs_plugin);
     mcs_plugin->plugin_name = g_strdup(PLUGIN_NAME);
-    mcs_plugin->caption = g_strdup(_("Mouse"));
+    /* the button label in the xfce-mcs-manager dialog */
+    mcs_plugin->caption = g_strdup (Q_ ("Button Label|Mouse"));
     mcs_plugin->run_dialog = run_dialog;
-    mcs_plugin->icon = xfce_themed_icon_load ("xfce4-mouse", 48);
     mcs_manager_notify(mcs_plugin->manager, CHANNEL1);
+    mcs_plugin->icon = xfce_themed_icon_load ("xfce4-mouse", 48);
+    if (G_LIKELY (mcs_plugin->icon != NULL))
+      g_object_set_data_full (G_OBJECT (mcs_plugin->icon), "mcs-plugin-icon-name", g_strdup ("xfce4-mouse"), g_free);
 
     return(MCS_PLUGIN_INIT_OK);
 }
